// This code example is created for educational purpose
// by Thorsten Thormaehlen (contact: www.thormae.de).
// It is distributed without any warranty.

package de.unimarburg.vbodrawarraysapp;

import android.app.Activity;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.os.SystemClock;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import javax.microedition.khronos.opengles.GL11;

import android.opengl.GLU;
import android.util.Log;
import android.view.MotionEvent;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

class Renderer {
  public float t = 0.0f;
  public int mode = 1;

  private int[] bufID = new int[1];
  private int bufSize = 0;
  private int SIZEOF_FLOAT = Float.SIZE / 8;

  public void init(GL10 gl10, EGLConfig config) {
    // this function corresponds to our init() function

    GL11 gl = (GL11) gl10;
    gl.glEnable(GL11.GL_DEPTH_TEST);

    // generating VBO input data
    int ayimutSegs = 100;
    int polarSegs = 100;
    ByteBuffer bb = ByteBuffer.allocateDirect(ayimutSegs * polarSegs * 4
        * SIZEOF_FLOAT);
    bb.order(ByteOrder.nativeOrder());
    FloatBuffer dataIn = bb.asFloatBuffer();

    double ayimutStep = 2.0f * Math.PI / (double) ayimutSegs;
    double polarStep = Math.PI / (double) polarSegs;
    double r = 1.0f;
    bufSize = 0;
    for (int m = 0; m < ayimutSegs; m++) {
      for (int n = 0; n < polarSegs; n++) {
        double phi = ayimutStep * m;
        double theta = polarStep * n;

        // compute xyz from spherical coordinates
        float x = (float) (r * Math.sin(theta) * Math.cos(phi));
        float y = (float) (r * Math.sin(theta) * Math.sin(phi));
        float z = (float) (r * Math.cos(theta));

        dataIn.put(x);
        dataIn.put(y);
        dataIn.put(z);
        dataIn.put(1.0f);
        bufSize++;
      }
    }

    dataIn.flip();

    gl.glGenBuffers(1, bufID, 0);
    gl.glBindBuffer(GL11.GL_ARRAY_BUFFER, bufID[0]);
    gl.glBufferData(GL11.GL_ARRAY_BUFFER, dataIn.capacity() * SIZEOF_FLOAT,
        dataIn, GL11.GL_STATIC_DRAW);
  }

  public void resize(GL10 gl10, int width, int height) {
    GL11 gl = (GL11) gl10;
    // corresponds to our resize() function
    gl.glViewport(0, 0, width, height);
    gl.glMatrixMode(GL11.GL_PROJECTION);
    gl.glLoadIdentity();
    GLU.gluPerspective(gl, 30.0f, (float) width / (float) height, 0.1f, 50.0f);
  }

  public void display(GL10 gl10) {
    GL11 gl = (GL11) gl10;
    // this function corresponds to our display() function

    gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    gl.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

    gl.glMatrixMode(GL11.GL_MODELVIEW);
    gl.glLoadIdentity();
    // set camera
    GLU.gluLookAt(gl10, 3.5f, -1.0f, 3.5f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
    // draw scene
    gl.glRotatef(t, 0.0f, 0.0f, 1.0f);

    // activating VBO
    gl.glBindBuffer(GL11.GL_ARRAY_BUFFER, bufID[0]);
    
    
    int stride = 0;
    gl.glVertexPointer(4, GL11.GL_FLOAT, stride, 0);
    gl.glEnableClientState(GL11.GL_VERTEX_ARRAY);

    gl.glBindBuffer(GL11.GL_ARRAY_BUFFER, bufID[0]);
  
    if(this.mode == 0) {
      gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    } else {
      // using the VBO geometry data for vertex colors as well
      gl.glColorPointer(4, GL11.GL_FLOAT, stride, 0);
      gl.glEnableClientState(GL11.GL_COLOR_ARRAY);
    }
    
    // render VBO
    gl.glDrawArrays(GL11.GL_POINTS, 0, bufSize);

    gl.glDisableClientState(GL11.GL_VERTEX_ARRAY);
    gl.glDisableClientState(GL11.GL_COLOR_ARRAY);

    gl.glFlush();
  }

}

class GLSurfaceViewRenderer implements GLSurfaceView.Renderer {

  private Renderer renderer = new Renderer();
  
  final int targetFPS = 60;
  final int targetMillis = 1000 / targetFPS;
  private long startTime = 0;
  
  @Override
  public void onDrawFrame(GL10 gl10) {
    
    float offset = 0.5f;
    renderer.t += offset;
    renderer.display(gl10);
    
    // this code attempts to achieve a target frames-per-second rate
    long timeTillNextRender = startTime + targetMillis;
    long sleepTime = timeTillNextRender - SystemClock.uptimeMillis();
    if (sleepTime > 0) {
      try {
        Thread.sleep(sleepTime);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    } else {
      // rendering is too slow
    }
    startTime = SystemClock.uptimeMillis();
  }

  @Override
  public void onSurfaceChanged(GL10 gl10, int width, int height) {
    renderer.resize(gl10, width, height);
  }

  @Override
  public void onSurfaceCreated(GL10 gl10, EGLConfig config) {
    renderer.init(gl10, config);
    startTime = SystemClock.uptimeMillis();
  }

  public Renderer getRenderer() {
    return renderer;
  }
}

public class VboDrawArrays extends Activity {

  private GLSurfaceView myGLSurfaceView;
  private GLSurfaceViewRenderer myGLSurfaceViewRenderer;
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    myGLSurfaceView = new GLSurfaceView(this);
    myGLSurfaceViewRenderer = new GLSurfaceViewRenderer();
    myGLSurfaceView.setRenderer(myGLSurfaceViewRenderer);
    // myGLSurfaceView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
    setContentView(myGLSurfaceView);
  }

  @Override
  protected void onPause() {
    // Large objects in graphics memory should be de-allocated here
    // This is skipped here for simplicity of the example.
    super.onPause();
    myGLSurfaceView.onPause();
  }

  public boolean onTouchEvent(MotionEvent event) {
    if(event.getAction() == MotionEvent.ACTION_DOWN) {
      if(myGLSurfaceViewRenderer.getRenderer().mode == 0) {
        myGLSurfaceViewRenderer.getRenderer().mode = 1;
      } else {
        myGLSurfaceViewRenderer.getRenderer().mode = 0;
      }
      return true; 
    }
    return false;
  }
  
  @Override
  protected void onResume() {
    super.onResume();
    // Large objects in graphics memory should be re-created here
    // if they were de-allocated on onPause().
    // This is skipped here for simplicity of the example.
    myGLSurfaceView.onResume();
  }
}
